@@ -161,7 +161,8 @@
 /*	policies[].policy.policy-string[]. Ignored if the tls_policy_type
 /*	value is TLSRPT_NO_POLICY_FOUND.
 /* .IP tls_policy_domain (may be null)
-/*	policies[].policy.policy-domain.
+/*	policies[].policy.policy-domain. If null, this defaults to the
+/*	TLSRPT policy domain.
 /* .IP mx_host_patterns (may be null)
 /*	policies[].policy.mx-host[]. Ignored if the tls_policy_type
 /*	value is TLSRPT_NO_POLICY_FOUND.
@@ -252,6 +253,13 @@
 #include <tlsrpt.h>
 #endif
 
+#if !defined(TLSRPT_PREREQ)
+#define TLSRPT_PREREQ(maj, min) \
+    (defined(TLSRPT_VERSION_MAJOR) && \
+	((TLSRPT_VERSION_MAJOR << 16) + TLSRPT_VERSION_MINOR >= \
+	    ((maj) << 16) + (min)))
+#endif
+
  /*
   * Utility library.
   */
@@ -312,8 +320,24 @@ TLSRPT_WRAPPER *trw_create(const char *rpt_socket_name,
 			           const char *rpt_policy_string,
 			           int skip_reused_hs)
 {
+    const char myname[] = "trw_create";
     TLSRPT_WRAPPER *trw;
 
+    if (msg_verbose > 1)
+	msg_info("%s(rpt_socket_name=%s, rpt_policy_domain=%s, "
+		 "rpt_policy_string=%s, skip_reused_hs=%d)",
+		 myname, rpt_socket_name, rpt_policy_domain,
+		 rpt_policy_string, skip_reused_hs);
+
+#if TLSRPT_PREREQ(0, 6)
+    if (tlsrpt_version_check(TLSRPT_VERSION_MAJOR, TLSRPT_VERSION_MINOR,
+			     TLSRPT_VERSION_PATCH) == 0)
+	msg_warn("run-time library vs. compile-time header version mismatch: "
+		 "libtlsrpt API version '%s' is not compatible with "
+		 "libtlsrpt API version '%s' ", tlsrpt_version(),
+		 TLSRPT_VERSION_STRING);
+#endif
+
     /*
      * memset() is not portable for pointer etc. types.
      */
@@ -338,6 +362,10 @@ TLSRPT_WRAPPER *trw_create(const char *rpt_socket_name,
 
 void    trw_free(TLSRPT_WRAPPER *trw)
 {
+    if (msg_verbose > 1)
+	msg_info("trw_free: rpt_socket_name=%s, rpt_policy_domain=%s, ...",
+		 trw->rpt_socket_name, trw->rpt_policy_domain);
+
     /* Destroy fields set with trw_create(). */
     myfree(trw->rpt_socket_name);
     myfree(trw->rpt_policy_domain);
@@ -362,11 +390,26 @@ void    trw_set_tls_policy(TLSRPT_WRAPPER *trw,
 			           const char *tls_policy_domain,
 			           const char *const * mx_host_patterns)
 {
+    const char myname[] = "trw_set_tls_policy";
+
+#define STR_OR_NULL(s)	((s) ? (s) : "(Null)")
+#define PSTR_OR_NULL(p)	((p) ? STR_OR_NULL(*p) : "(Null)")
+
+    if (msg_verbose > 1)
+	msg_info("%s(tlsrpt_policy_type_t=%d, tls_policy_strings=%s..., "
+		 "tls_policy_domain=%s, mx_host_patterns=%s...)",
+		 myname, tls_policy_type,
+		 PSTR_OR_NULL(tls_policy_strings),
+		 STR_OR_NULL(tls_policy_domain),
+		 PSTR_OR_NULL(mx_host_patterns));
+
     trw->tls_policy_type = tls_policy_type;
+    if (tls_policy_domain == 0)
+	tls_policy_domain = trw->rpt_policy_domain;
     MYFREE_IF_SET_AND_COPY(trw->tls_policy_domain, tls_policy_domain);
     if (tls_policy_type == TLSRPT_NO_POLICY_FOUND) {
 	ARGV_FREE_IF_SET_AND_CLEAR(trw->tls_policy_strings);
-	ARGV_FREE_IF_SET_AND_CLEAR(trw->tls_policy_strings);
+	ARGV_FREE_IF_SET_AND_CLEAR(trw->mx_host_patterns);
     } else {
 	ARGV_FREE_IF_SET_AND_COPY(trw->tls_policy_strings, tls_policy_strings);
 	ARGV_FREE_IF_SET_AND_COPY(trw->mx_host_patterns, mx_host_patterns);
@@ -385,6 +428,11 @@ void    trw_set_tcp_connection(TLSRPT_WRAPPER *trw,
 {
     const char myname[] = "trw_set_tcp_connection";
 
+    if (msg_verbose > 1 && (snd_mta_addr || rcv_mta_name || rcv_mta_addr))
+	msg_info("%s(snd_mta_addr=%s, rcv_mta_name=%s, rcv_mta_addr=%s)",
+		 myname, STR_OR_NULL(snd_mta_addr),
+		 STR_OR_NULL(rcv_mta_name), STR_OR_NULL(rcv_mta_addr));
+
     /*
      * Sanity check: usage errors are not a show stopper.
      */
@@ -405,6 +453,9 @@ void    trw_set_ehlo_resp(TLSRPT_WRAPPER *trw, const char *rcv_mta_ehlo)
 {
     const char myname[] = "trw_set_ehlo_resp";
 
+    if (msg_verbose > 1 && rcv_mta_ehlo)
+	msg_info("%s(rcv_mta_ehlo=%s)", myname, rcv_mta_ehlo);
+
     /*
      * Sanity check: usage errors are not a show stopper.
      */
@@ -499,6 +550,11 @@ int     trw_report_failure(TLSRPT_WRAPPER *trw,
     struct tlsrpt_connection_t *con;
     int     res;
 
+    if (msg_verbose > 1)
+	msg_info("%s(failure_type=%d, additional_info=%s, failure_reason=%s)",
+		 myname, failure_type, STR_OR_NULL(additional_info),
+		 STR_OR_NULL(failure_reason));
+
     /*
      * Sanity check: usage errors are not a show stopper.
      */
@@ -579,6 +635,9 @@ int     trw_report_success(TLSRPT_WRAPPER *trw)
     struct tlsrpt_connection_t *con;
     int     res;
 
+    if (msg_verbose > 1)
+	msg_info("trw_report_success");
+
     /*
      * Sanity check: usage errors are not a show stopper.
      */
