@@ -108,5 +108,5 @@ DICT   *dict_env_open(const char *name, int unused_flags, int dict_flags)
     if (dict_flags & DICT_FLAG_FOLD_FIX)
 	dict->fold_buf = vstring_alloc(10);
     dict->owner.status = DICT_OWNER_TRUSTED;
-    return (DICT_DEBUG (dict));
+    return (dict);
 }
